{===
Flyer v1.0, a demo for approximated equation solution.
Copyright (C) 2009 Marco Borsari

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

You may contact me at address <borsa77@libero.it> or you can
compile by yourself with Turbo Pascal 5.5+ or Free Pascal 2.0.4+.
===}

Program Flyer;
{$R-,S-,N+,E+}
{$IFDEF go32v2}
Uses emu387;
{$ENDIF}

Const prec = 0.00001;
  R = 0.0820578;
  T = 298.15;
  nr = 2;

Type float = extended;
  ls = array[0..nr] of float;

Var P,n,x1,x2: float;
  step: byte;
  wa,wb,mw: ls;
  cf: array[0..3] Of float;
  sp: byte;
  o: char;

Function f3(x:float): float; (*metodo di Horner*)
Var t: float;
  i: byte;
Begin
  t := cf[0]*x;
  For i:=1 To 2 Do
    t := x*(t+cf[i]);
  t := t+cf[3];
  f3 := t;
End;

Procedure Bisezione;
Var xm,y1,y2,ym: float;
  sgn1,sgn2,sgnm: integer;

Function sgn(y:float): integer;
Begin
  If y>=0 Then sgn := 1
  Else sgn := -1;
End;

Begin
  y1 := f3(x1);
  sgn1 := sgn(y1);
  y2 := f3(x2);
  sgn2 := sgn(y2);
  While x2-x1>prec Do
    If y1=0 Then
      x2 := x1
    Else If y2=0 Then
      x1 := x2
    Else If sgn1<>sgn2 Then
      Begin
        step := step+1;
        xm := (x1+x2)/2.0;
        ym := f3(xm);
        sgnm := sgn(ym);
        If sgn1=sgnm Then
          Begin
            x1 := xm;
            y1 := ym;
            sgn1 := sgnm;
          End
        Else
          Begin
            x2 := xm;
            y2 := ym;
            sgn2 := sgnm;
          End;
      End
    Else
      Begin
        writeln('Sign retention on both halves.');
        exit;
      End;
  If x2>2.0*prec Then writeln('Result (L): ',x1*n:6:2)
  Else writeln('Insufficient precision.');
End;

Procedure Newton;
Var od,xd,xt: float;

Function d3(x:float): float;
Var t: float;
Begin
  t := 3.0*cf[0]*x;
  t := x*(t+2.0*cf[1])+cf[2];
  d3 := t;
End;

Begin
  od := x2-x1;
  xd := od;
  xt := (x1+x2)/2.0;
  If xd>prec Then
    Begin
      While xd>prec Do
        If (xd<=od) And (xt>x1) And (xt<x2) Then
          Begin
            step := step+1;
            od := xd;
            xd := d3(xt);
            If xd<>0 Then
              Begin
                xd := f3(xt)/xd;
                xt := xt-xd;
              End
            Else
              Begin
                writeln('Null derivative.');
                exit;
              End;
            xd := abs(xd);
          End
        Else
          Begin
            writeln('Divergence.');
            exit;
          End;
      writeln('Result (L): ',xt*n:6:2);
    End
  Else writeln('Insufficient precision.');
End;

Begin
  writeln('Flyer v1.0, Copyright (C) 2009, Marco Borsari');
  writeln('Flyer comes with ABSOLUTELY NO WARRANTY.');
  writeln('This is free software, and you are welcome');
  writeln('to redistribute it under the terms of GNU GPL.');
  writeln;
  wa[1]:=2.283;
  wb[1]:=0.04278;
  mw[1]:=12.011+4*1.008;
  wa[2]:=3.640;
  wb[2]:=0.04267;
  mw[2]:=12.011+2*15.999;
  writeln('Available gas species');
  writeln('0 - your own');
  writeln('1 - methane');
  writeln('2 - carbon dioxide');
  Repeat
    write('Please select: ');
    readln(sp);
  Until sp<=nr;
  If sp=0 Then
    Begin
      write('van der Waals a (L^2bar/mol^2): ');
      readln(wa[sp]);
      write('van der Waals b (L/mol): ');
      readln(wb[sp]);
      write('Molecular weight (u): ');
      readln(mw[sp]);
    End;
  writeln('Critical pressure (bar): ',wa[sp]/(27.0*sqr(wb[sp])):5:1);
  writeln('Critical volume (L): ',3.0*wb[sp]:6:2);
  P := 1.0;
  write('Do you want to leave the default pressure (',P:5:1,' bar) (y/n)? ');
  readln(o);
  If o='n' Then
    Begin
      write('Pressure (bar): ');
      readln(P);
    End;
  P := P/1.01325; (*bar->atm*)
  write('Weight (Kg): ');
  readln(n);
  n := n*1000.0/mw[sp]; (*Kg->mol*)
  cf[0] := 1.0;
  cf[1] := -(wb[sp]+R*T/P);
  cf[2] := wa[sp]/P;
  cf[3] := -cf[2]*wb[sp]; (*equazione di van der Waals*)
  x1 := prec;
  x2 := 2.0*R*T/P;
  write('Do you want to leave the Vm presumed for the ideal gas (',x2:4:1,' L/mol) (y/n)? ');
  readln(o);
  If o='n' Then
    Begin
      write('Vm range (L/mol): ');
      readln(x2);
    End;
  step := 0;
  write('Do you want to apply the fast method (y/n)? ');
  readln(o);
  If o='n' Then Bisezione
  Else Newton;
  writeln('Cycles to compute: ',step);
End.
