{===
MrCircus v2.3, a plain Gauss system solver.
Copyright (C) 1999 Marco Borsari

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

You may contact me at address <borsa77@libero.it> or you can
compile by yourself with Turbo Pascal 5.5+ or Free Pascal 2.0.4+.
===}

PROGRAM MrCircus;
{$R-,S-,N+,E+}
{$IFDEF go32v2}
USES Emu387;
{$ENDIF}
CONST
  nmax       = 100;
  sts : BYTE = 0;
TYPE
  tipor = EXTENDED;
  vett  = ARRAY[1..nmax+1] OF tipor;
  vettd = ^vett;
  matr  = ARRAY[1..nmax] OF vettd;
  matrd = ^matr;
VAR
  nf : STRING[20];
  { </ SIGNIFICATO DELLE VARIABILI:             }
  { n - numero delle equazioni, limitato a nmax }
  { A - matrice dei coefficienti                }
  { b - vettore dei termini noti                }
  { x - vettore delle incognite                 }
  { S - matrice a n+1 colonne per includere A+b }
  n : INTEGER;
  S : matrd;
  { />}

PROCEDURE Guida;
{ routine per l'aiuto a linea di comando }
BEGIN
  WRITELN('	HELP ON LINE');
  WRITELN('This `bare-bones'' program solves a system of linear');
  WRITELN('equations, first you have to store it on the disk so');
  WRITELN('you make an input file with extension SYS as follows:');
  WRITELN('SYSTEM SAMPLE');
  WRITELN('X(1) - 2.5X(2) = -6');
  WRITELN('X(1) + 1.5X(2) =  2');
  WRITELN('FILE MRCIRCUS.SYS');
  WRITELN('line 1) #abc <- comment (in any line)');
  WRITELN('line 2) 2    <- number of equations or unknowns');
  WRITELN('line 3) 1    <- coefficient of X(1) first equation');
  WRITELN('line 4) -2.5 <- coefficient of X(2) first equation');
  WRITELN('line 5) -6   <- first constant');
  WRITELN('line 6) 1    <- coefficient of X(1) second equation');
  WRITELN('line 7) 1.5  <- coefficient of X(2) second equation');
  WRITELN('line 8) 2    <- second constant');
  WRITELN('Launching ~mrcircus [-t] [FILE]~ you may choose the');
  WRITELN('name of the input (default CAKE) which will even');
  WRITELN('be the output with extension SOL.');
  HALT(0);
END;

PROCEDURE CreaMatr(VAR m : matrd; dim : INTEGER);
{ routine per l'allocazione dinamica con controllo sull'heap }
VAR
  i       : INTEGER;
  sp1,sp2 : LONGINT;
BEGIN
  sp1:=dim*SIZEOF(vettd);
  sp2:=(dim+1)*SIZEOF(tipor);
{$IFDEF ver70}
  IF sp1+dim*sp2>MAXAVAIL THEN
    BEGIN
      sts:=7;
      m:=NIL;
    END;
{$ENDIF}
{$IFDEF fpc}
  RETURNNILIFGROWHEAPFAILS:=TRUE;
{$ENDIF}
  IF sts<2 THEN
    BEGIN
      i:=1;
      GETMEM(m,sp1);
      IF m<>NIL THEN
        REPEAT
          GETMEM(m^[i],sp2);
          IF m^[i]<>NIL THEN FILLCHAR(m^[i]^,sp2,0)
          ELSE sts:=7;
          INC(i);
        UNTIL (i>dim) OR (sts>1)
      ELSE sts:=7;
    END;
END;

PROCEDURE Opzioni;
{ routine per la selezione delle opzioni }
VAR
  stg : STRING[20];
  p,q : BYTE;
BEGIN
  WRITELN('MrCircus v2.3, Copyright (C) 1999 Marco Borsari');
  WRITELN('MrCircus comes with ABSOLUTELY NO WARRANTY.');
  WRITELN('This is free software, and you are welcome');
  WRITELN('to redistribute it under the terms of GNU GPL.');
  WRITELN;
  nf:='cake';
  FOR p:=1 TO PARAMCOUNT DO
    BEGIN
      stg:=PARAMSTR(p);
      IF stg[1]='-' THEN
        BEGIN
          DELETE(stg,1,1);
          FOR q:=1 TO LENGTH(stg) DO stg[q]:=UPCASE(stg[q]);
          IF stg='H' THEN Guida
          ELSE IF stg='T' THEN sts:=1;
        END
      ELSE
        BEGIN
          q:=POS('.',stg);
          IF q>0 THEN nf:=COPY(stg,1,q-1)
          ELSE nf:=stg;
        END;
    END;
END;

PROCEDURE Carica(VAR n : INTEGER; VAR S : matrd);
{ routine per caricare il sistema dal disco in memoria }
TYPE
  tipos = STRING[100];
VAR
  f : TEXT;

  FUNCTION Riga : tipos;
  { routine per leggere le righe dal file di testo }
  VAR
    stg   : tipos;
    i,j,l : BYTE;
  BEGIN
    REPEAT
      IF SEEKEOF(f) THEN
        BEGIN
          sts:=4;
          Riga:='0';
        END
      ELSE
        BEGIN
          READLN(f,stg);
          { </ elimina gli spazi bianchi che danno errore con VAL }
          l:=LENGTH(stg);
          i:=1;
          j:=l;
          WHILE (i<l) AND ((stg[i]=#9) OR (stg[i]=#32)) DO INC(i);
          WHILE (j>i) AND ((stg[j]=#9) OR (stg[j]=#32)) DO DEC(j);
          Riga:=COPY(stg,i,j-i+1);
          { />}
        END;
    { </ salta le linee che iniziano con # come commenti }
    UNTIL (stg[1]<>'#') OR (sts>1);
    { />}
  END;

  PROCEDURE Inizia(VAR n : INTEGER; VAR S : matrd);
  { routine di inizializzazione, n e S passati per indirizzo }
  VAR
    i,j,cod : INTEGER;
  BEGIN
    { </ qui i e' usato anche come check prima di assegnare n, }
    { se c'e' un fine file da Riga n viene riassegnato a zero  }
    VAL(Riga,i,cod);
    IF cod>0 THEN sts:=5
    ELSE IF (i<0) OR (i>nmax) THEN sts:=3
    ELSE n:=i;
    { />}
    CreaMatr(S,n);
    FOR i:=1 TO n DO
      BEGIN
        j:=1;
        WHILE (j<=n+1) AND (sts<2) DO
          BEGIN
            VAL(Riga,S^[i]^[j],cod);
            IF cod>0 THEN sts:=5;
            INC(j);
          END;
      END;
  END;

BEGIN
  n:=0;
{$I-}
  ASSIGN(f,nf+'.sys');
  RESET(f);
{$I+}
  IF IORESULT=0 THEN
    BEGIN
      Inizia(n,S);
      CLOSE(f);
    END
  ELSE sts:=2;
END;

PROCEDURE Fattorizza(n : INTEGER; S : matrd);
{ routine della fattorizzazione per trasformazioni elementari di Gauss }
VAR
  i,j,k  : INTEGER;
  m,prec : tipor;

  FUNCTION EPSM : tipor;
  { routine di calcolo dello zero macchina per il tipo prescelto }
  VAR
    x,y : tipor;
  BEGIN
    x:=1.0;
    y:=1.0+x;
    WHILE (y>1.0) AND (sts<2) DO
      BEGIN
        x:=x/2.0;
        y:=1.0+x;
      END;
    EPSM:=x;
  END;

  PROCEDURE Pivot(n,j : INTEGER; S : matrd);
  { routine di ricerca massimo elemento diagonale sulle righe }
  VAR
    aus    : vettd;
    i,magg : INTEGER;
  BEGIN
    magg:=j;
    FOR i:=j+1 TO n DO
      IF ABS(S^[i]^[j])>ABS(S^[magg]^[j]) THEN magg:=i;
    aus:=S^[j];
    S^[j]:=S^[magg];
    S^[magg]:=aus;
  END;

BEGIN
  k:=1;
  prec:=EPSM;
  WHILE (k<=n) AND (sts<2) DO
    BEGIN
      Pivot(n,k,S);
      IF ABS(S^[k]^[k])<=prec THEN sts:=6
      ELSE FOR i:=k+1 TO n DO
        BEGIN
          m:=S^[i]^[k]/S^[k]^[k];
          FOR j:=k TO n+1 DO S^[i]^[j]:=S^[i]^[j]-m*S^[k]^[j];
        END;
      INC(k);
    END;
END;

PROCEDURE Risolvi(n : INTEGER; S : matrd);
{ routine della soluzione del sistema triangolare }
VAR
  pol : tipor;
  i,j : INTEGER;
BEGIN
  IF sts<2 THEN
    FOR i:=n DOWNTO 1 DO
      BEGIN
        pol:=0.0;
        { </ qui i vettori b e x usano entrambi la colonna n+1 }
        { perche' non si sovrappongono                         }
        FOR j:=i+1 TO n DO pol:=pol+S^[i]^[j]*S^[j]^[n+1];
        S^[i]^[n+1]:=(S^[i]^[n+1]-pol)/S^[i]^[i];
        { />}
      END;
END;

PROCEDURE Salva(n : INTEGER; S : matrd);
{ routine per salvare la soluzione su disco }
CONST
  cfr = 10;
  stg = #13#10+'try the -h option for more information.';
VAR
  i,j : INTEGER;
  f   : TEXT;
BEGIN
  CASE sts OF
    0,1 : BEGIN
            ASSIGN(f,nf+'.sol');
            REWRITE(f);
            WRITELN(f,'Solution:');
            FOR i:=1 TO n DO WRITELN(f,'X(',i,')= ',S^[i]^[n+1]:1:cfr);
            CLOSE(f);
            { </ l'opzione -t salva la matrice triangolare }
            IF sts=1 THEN
              BEGIN
                ASSIGN(f,nf+'.trm');
                REWRITE(f);
                FOR i:=1 TO n DO
                  BEGIN
                    FOR j:=1 TO n DO WRITE(f,S^[i]^[j],' ');
                    WRITELN(f);
                  END;
                CLOSE(f);
              END;
            { />}
            WRITELN('We hope you have enjoyed the show!');
          END;
    2   : WRITELN('Invalid file name,',stg);
    3   : WRITELN('Number of equations out of (0-',nmax,'),',stg);
    4   : WRITELN('Unexpected end in the input file,',stg);
    5   : WRITELN('Syntax error in the input file,',stg);
    6   : WRITELN('There is a singular matrix.');
    7   : WRITELN('Not enough memory to run.');
  END;
END;

BEGIN
  Opzioni;
  Carica(n,S);
  Fattorizza(n,S);
  Risolvi(n,S);
  Salva(n,S);
END.
